@@ -5,16 +5,69 @@ This module imports pandas and numpy, the arithmetic.py module should not
 """
 import numpy as np
 import pandas as pd
+import geopandas as gpd
 from dweather_client.http_client import get_rainfall_dict, get_rev_rainfall_dict, get_heads, get_metadata
 from dweather_client.utils import listify_period, get_n_closest_station_ids
-from dweather_client.ipfs_client import cat_station_df, cat_station_csv, cat_metadata
+from dweather_client.ipfs_client import cat_rainfall_dict, cat_station_df, cat_station_csv, cat_metadata
 from dweather_client.df_utils import get_station_ids_with_icao
 import ipfshttpclient
 import datetime
 import logging
+import os
+from dweather_client.utils import snap_to_grid
+from dweather_client.http_client import get_metadata, get_rainfall_dict
+from shapely.geometry import Point, Polygon
+import ipfshttpclient
+from collections import Counter
 
 HISTORICAL_START_YEAR = 1981
 
+def get_polygon_df(shapefile_path, dataset, polygon_names, bounding_box, encoding='UTF-8'):
+    """
+    Get a dataframe of climate data for a given set of polygons.
+    """
+    polygons = gpd.read_file(shapefile_path)[['NAME_0', 'NAME_1', 'geometry']]
+    polygons.columns = ["country", "state", "geometry"]
+
+    metadata = cat_metadata(get_heads()[dataset])
+
+    start = datetime.datetime.strptime(metadata['date range'][0], '%Y/%d/%m')
+    end = datetime.datetime.today()
+    date_range = pd.date_range(start=start, end=end)
+    df = pd.DataFrame([[0 for i in polygon_names] for i in date_range], columns=polygon_names)
+    df.insert(0, "Date", date_range, True)
+    df.index = df['Date']
+    df.drop(['Date'], axis=1, inplace=True)
+    df.index.name = None
+
+    with ipfshttpclient.connect() as client:
+        for index, row in polygons.iterrows():
+            bbox_size = int((bounding_box[1].y - bounding_box[0].y) * (bounding_box[1].x - bounding_box[0].x) / metadata['resolution'] / metadata['resolution'])
+            logging.info("Building %s (%i of %i polygons)" % (row['state'], index + 1, len(polygons.index)))
+            logging.info("Matching points within a bounding box of %i points." % bbox_size)
+            exec_counter = 0
+            poly_counter = Counter({})
+            for latitr in np.arange(bounding_box[0].y, bounding_box[1].y, metadata['resolution']):
+                if ((exec_counter % 1000) == 0):
+                    logging.info("Scanning point %i of %i" % (exec_counter, bbox_size))
+                for lonitr in np.arange(bounding_box[0].x, bounding_box[1].x, metadata['resolution']):
+                    if Point(lonitr, latitr).within(row['geometry']):
+                        slat, slon = snap_to_grid(latitr, lonitr, metadata)
+                        logging.info("Executing IPFS cat.")
+                        rain_counter = get_rainfall_dict(slat, slon, dataset, get_counter=True) #, client=client)
+                        logging.info("Found match at (%s, %s), point %i of %i" % ( \
+                            "{:.3f}".format(slat), 
+                            "{:.3f}".format(slon),
+                            exec_counter,
+                            bbox_size
+                        ))
+                        poly_counter = poly_counter + rain_counter
+                    exec_counter = exec_counter + 1
+            for day in poly_counter:
+                df.at[day.strftime('%Y-%m-%d'), row['state']] += poly_counter[day]
+    return df
+ 
+
 def cat_station_df_list(station_ids, pin=True, force_hash=None):
     batch_hash = force_hash
     if (force_hash is None):
